#!/bin/bash

# Installation et informations sur Easy2boot:
# https://wiki.gardouille.fr/doku.php?id=documentation:administration_systeme:easy2boot

dl_iso() {
  ## URL to download
  URL="${1}"
  ## Path to store the downloaded file
  ISO_PATH="${2}"
  ## Date 30 days ago
  DATE=$(date --date '30 days ago' +%Y-%m-%d)

  # If the file doesn't already exist and not older than 30 days
	if [ ! "$(find . -samefile "${ISO_PATH}" -newerct "${DATE}")" ]; then
    wget "${URL}" -O "${ISO_PATH}"
  fi
}

#### Debian
DEBIAN_STABLE="7.5.0"
## Standard Stable versions
# AMD64 (~250Mo)
dl_iso "http://cdimage.debian.org/debian-cd/${DEBIAN_STABLE}/amd64/iso-cd/debian-${DEBIAN_STABLE}-amd64-netinst.iso" "LINUX/debian-${DEBIAN_STABLE}-amd64-netinst.iso"
# i386 (~200Mo)
dl_iso "http://cdimage.debian.org/debian-cd/${DEBIAN_STABLE}/i386/iso-cd/debian-${DEBIAN_STABLE}-i386-netinst.iso" "LINUX/debian-${DEBIAN_STABLE}-i386-netinst.iso"

## Version with firmares
# AMD64 (~300Mo)
dl_iso "http://cdimage.debian.org/cdimage/unofficial/non-free/cd-including-firmware/${DEBIAN_STABLE}/amd64/iso-cd/firmware-${DEBIAN_STABLE}-amd64-netinst.iso" "LINUX/firmware-${DEBIAN_STABLE}-amd64-netinst.iso"
# i386 (~250Mo)
dl_iso "http://cdimage.debian.org/cdimage/unofficial/non-free/cd-including-firmware/${DEBIAN_STABLE}/i386/iso-cd/firmware-${DEBIAN_STABLE}-i386-netinst.iso" "LINUX/firmware-${DEBIAN_STABLE}-i386-netinst.iso"

#### ArchLinux (~550Mo)
dl_iso "http://archlinux.mirrors.ovh.net/archlinux/iso/2014.05.01/archlinux-2014.05.01-dual.iso" "LINUX/archlinux-2014.05.01-dual.iso"


#### Tails (~1Go)
dl_iso "http://dl.amnesia.boum.org/tails/stable/tails-i386-1.0/tails-i386-1.0.iso" "LINUX/tails-i386-1.0.iso"

#### Kali (~2,9Go)
#dl_iso "http://cdimage.kali.org/kali-latest/amd64/kali-linux-1.0.6-amd64.iso" "LINUX/kali-linux-1.0.6-amd64.iso"


#### Clonezilla (~150Mo)
CLONEZILLA_STABLE="2.2.3-10"
dl_iso "http://downloads.sourceforge.net/project/clonezilla/clonezilla_live_stable/${CLONEZILLA_STABLE}/clonezilla-live-${CLONEZILLA_STABLE}-amd64.iso" "BACKUP_LINUX/clonezilla-live-${CLONEZILLA_STABLE}-amd64.iso"
dl_iso "http://downloads.sourceforge.net/project/clonezilla/clonezilla_live_stable/${CLONEZILLA_STABLE}/clonezilla-live-${CLONEZILLA_STABLE}-i686-pae.iso" "BACKUP_LINUX/clonezilla-live-${CLONEZILLA_STABLE}-i686-pae.iso"


#### Kaspersky (~200Mo)
dl_iso "http://rescuedisk.kaspersky-labs.com/rescuedisk/updatable/kav_rescue_10.iso" "ANTIVIRUS/kaspersky_anti_virus_rescue_cd_10.iso"

#### Gparted (~200Mo)
dl_iso "http://downloads.sourceforge.net/project/gparted/gparted-live-stable/0.18.0-2/gparted-live-0.18.0-2-i486.iso" "UTILITIES/gparted-live-0.18.0-2-i486.iso"


#### SystemrescueCD (~400Mo)
dl_iso "http://downloads.sourceforge.net/project/systemrescuecd/sysresccd-x86/4.2.0/systemrescuecd-x86-4.2.0.iso" "UTILITIES/systemrescuecd-x86-4.2.0.iso"

#### Memtest
dl_iso "http://www.memtest.org/download/5.01/memtest86+-5.01.iso.zip" "UTILITIES/memtest86+-5.01.iso.zip"
unzip UTILITIES/memtest86+-5.01.iso.zip -d UTILITIES/
rm -f UTILITIES/memtest86+-5.01.iso.zip

#### Bootdisk // Reset windows password
## http://pogostick.net/~pnh/ntpasswd/
dl_iso "http://pogostick.net/~pnh/ntpasswd/cd140201.zip" "UTILITIES/cd140201.zip"
unzip UTILITIES/cd140201.zip -d UTILITIES/
rm -f  UTILITIES/cd140201.zip
